@@ -2,13 +2,14 @@
 Description: Sample decoder
 Author: DC3
 """
-import re
 
 import kordesii
-from kordesii.utils import decoderutils
+from kordesii.utils import EncodedString
+from kordesii.utils import ida_re
 from kordesii.utils import function_tracing
 
 logger = kordesii.get_logger()
+emulator = function_tracing.Emulator()
 
 
 def xor_decrypt(key, enc_data):
@@ -25,15 +26,14 @@ def find_strings():
                 *s++ ^= key;
         }
     """
-    for encrypt_func in decoderutils.re_find_functions(re.compile(br"\x8b\x45\x08\x0f\xbe\x08")):
-        logger.info("Found XOR encrypt function at: 0x{:0x}".format(encrypt_func.start_ea))
-        for call_ea in encrypt_func.xrefs_to:
-            logger.debug("Tracing {:0x}".format(call_ea))
+    for encrypt_func in ida_re.find_functions(br"\x8b\x45\x08\x0f\xbe\x08"):
+        logger.info("Found XOR encrypt function at: 0x%x", encrypt_func.start_ea)
+        for call_ea in encrypt_func.calls_to:
+            logger.debug("Tracing 0x%08x", call_ea)
             # Extract arguments for call to xor function.
-            tracer = function_tracing.get_tracer(call_ea)
-            context, args = tracer.get_function_args(call_ea)
+            context, args = emulator.get_function_args(call_ea)
             enc_str_ptr, key = args
-            encoded_string = decoderutils.EncodedString(enc_str_ptr, string_reference=call_ea, key=key)
+            encoded_string = EncodedString(enc_str_ptr, string_reference=call_ea, key=key)
             encoded_string.decoded_data = xor_decrypt(key, encoded_string.encoded_data)
             encoded_string.publish(rename=True, patch=False)
 
