@@ -5,5 +5,5 @@ Provides user-facing API for kordesii.utils
 # Expose common classes and functions
 from .flowchart import Flowchart, Flowchart, BasicBlock
 from .functions import Function
-from .strings import EncodedString, EncodedStackString
+from .strings import EncodedString, EncodedStackString, find_string_data
 from .utils import *
